'use strict';

export { validatorsPropCheck };
export { makeValidators };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

import { getInverseMarker } from './markers';
import { line } from './strings';
import { filteredMap, getIsRequiredChecker } from './utils';

var validatorDefaults = {
    key: 'general',
    mark: true,
    className: 'txr-invalid'
};

export { validatorDefaults };
var PROP_TYPE_ERR_MESSAGE = line('\n    Invalid "validators" prop, must be either a RegExp, function, config\n    Object, or an Array of these. The config Object can have the optional\n    property \'className: <string>\' which would specify the className the\n    validators the Object defines would mark characters with. The config Object\n    may also have a \'mark: <boolean>\' property which, if false, will prevent the\n    validators specified by the config object from marking invalid text with a\n    className. The config Object may have an arbitrary number of key/value pairs\n    where the value should be a RegExp (must be a full-match) or a function\n    returning a boolean (true indicates a failure) and the key is the string\n    added to the [data-violations] attribute on the wrapping <div> if the RegExp\n    or function fail. Note: if a function returns a value other than a boolean\n    the value is converted to a string and added to the [data-violations]\n    attribute.');

function propValueCheck(value) {
    if (!value) {
        return 'Invalid value: ' + value;
    }
    if (typeof value === 'function') {
        return;
    }
    if (value instanceof RegExp) {
        return;
    }
    if (Array.isArray(value)) {
        return value.map(propValueCheck).filter(Boolean).join('; ');
    }
    if (typeof value !== 'object') {
        return 'Invalid value: ' + value;
    }
    if ('mark' in value && typeof value.mark !== 'boolean') {
        return 'The "mark" property must be a boolean.';
    }
    if ('className' in value && typeof value.className !== 'string') {
        return 'The "className" property must be a string.';
    }
    return Object.keys(value).map(function (key) {
        if (key !== 'className' && key !== 'mark' && !(value[key] instanceof RegExp) && typeof value[key] !== 'function') {
            return 'Invalid property, ' + key + ' must be a RegExp or function.';
        };
    }).filter(Boolean).join('; ');
}

function validatorsPropCheck(props, propName) {
    var value = props[propName];
    if (value == null) {
        return;
    }
    var msgs = propValueCheck(value);
    if (msgs) {
        return new Error(msgs + ' - ' + PROP_TYPE_ERR_MESSAGE);
    }
}

validatorsPropCheck.isRequired = getIsRequiredChecker(validatorsPropCheck);

// Different forms the validator definition can take (in JSX):
//
//         validators={/\d+/}
//         validators={myTestFn}
//         validators={{digits: /\d+/, mark: false}}
//         validators={{upperLetters: /[A-Z]+/, lowerLetters: /[a-z]+/}}
//         validators={[
//             /\d+/,
//             myTestFn,
//             {testKey: myTestFn},
//             {digits: /\d+/, mark: false},
//             {upperLetters: /[A-Z]+/, lowerLetters: /[a-z]+/, className: 'invalid'},
//         ]}

function makeValidators(value, invalidClassName) {
    invalidClassName = invalidClassName || validatorDefaults.className;
    switch (true) {
        case !value:
            throw new Error('Invalid validator value');

        case value instanceof RegExp:
            return new RxValidator(value, invalidClassName);

        case typeof value === 'function':
            return new FnValidator(value);

        case Array.isArray(value):
            var validatorArr = value.map(function (item) {
                return makeValidators(item, invalidClassName);
            });
            return new ValidatorSeries(validatorArr);

        case typeof value === 'object':
            var mark = value.mark,
                className = value.className || invalidClassName,
                keys = Object.keys(value).filter(function (key) {
                return key !== 'mark' && key !== 'className';
            }),
                validators = keys.map(function (key) {
                var v = value[key];
                if (v instanceof RegExp) {
                    return new RxValidator(v, className, key, mark);
                } else if (typeof v === 'function') {
                    return new FnValidator(v, key);
                } else {
                    throw new Error('Invalid validator value');
                }
            });
            return new ValidatorSeries(validators);
    }
}

var RxValidator = (function () {
    function RxValidator(rx) {
        var className = arguments.length <= 1 || arguments[1] === undefined ? validatorDefaults.className : arguments[1];
        var key = arguments.length <= 2 || arguments[2] === undefined ? validatorDefaults.key : arguments[2];
        var mark = arguments.length <= 3 || arguments[3] === undefined ? validatorDefaults.mark : arguments[3];

        _classCallCheck(this, RxValidator);

        this.rx = rx;
        this.key = key;
        if (mark) {
            this.marker = getInverseMarker(this.rx, className);
        } else {
            this.marker = undefined;
        }
    }

    RxValidator.prototype.exec = function exec(text) {
        var markRange = arguments.length <= 1 || arguments[1] === undefined ? undefined : arguments[1];

        var ok = !text.replace(this.rx, '').length;
        if (ok) {
            return;
        }
        if (this.marker && markRange) {
            this.marker(text, markRange);
        }
        return this.key;
    };

    return RxValidator;
})();

var FnValidator = (function () {
    function FnValidator(fn) {
        var key = arguments.length <= 1 || arguments[1] === undefined ? validatorDefaults.key : arguments[1];

        _classCallCheck(this, FnValidator);

        this.fn = fn;
        this.key = key;
    }

    FnValidator.prototype.exec = function exec(text, markRange) {
        var rv = this.fn(text, markRange);
        if (rv) {
            return typeof rv === 'boolean' ? this.key : '' + rv;
        }
    };

    return FnValidator;
})();

var ValidatorSeries = (function () {
    function ValidatorSeries(validators) {
        _classCallCheck(this, ValidatorSeries);

        this.validators = validators;
    }

    ValidatorSeries.prototype.exec = function exec(text, markRange) {
        return filteredMap(this.validators, function (v) {
            return v.exec(text, markRange);
        }).join(' ');
    };

    return ValidatorSeries;
})();